@@ -34,7 +34,7 @@ _local_modules = [model_pb2, model_reference_pb2, standard_sql_pb2]
 
 names = []
 
-for module in _shared_modules:
+for module in _shared_modules:  # pragma: NO COVER
     for name, message in get_messages(module).items():
         setattr(sys.modules[__name__], name, message)
         names.append(name)
